@@ -134,7 +134,7 @@ static gboolean spyshutdown = FALSE;/* this global could be removed if we have a
 						way of removing our idle handler in gok_spy_stop */
 
 #define MAX_CHILD_COUNT 100
-
+#define MAX_NAME_SEARCH_DEPTH 10
 
 static void
 gok_spy_set_focussed_object (Accessible *accessible)
@@ -720,34 +720,53 @@ gok_spy_get_list_parent (Accessible *search_root)
 }
 
 static gchar*
-gok_spy_concatenate_child_names (Accessible *parent)
+gok_spy_concatenate_child_names (Accessible *parent, gint depth)
 {
 	gint num_children = 0;
 	gchar *tmp = NULL, *name = NULL;
+
+	/* avoid cyclic graph */
+	if (depth++ >= MAX_NAME_SEARCH_DEPTH) 
+	{
+		return NULL;
+	}
+		
 	if (parent) 
 	{
 		gint i;
 		num_children = Accessible_getChildCount (parent);
-		for (i = 0; i < num_children; ++i) 
+		for (i = 0; i < num_children; ++i)  
 		{
 			Accessible *child = Accessible_getChildAtIndex (parent, i);
 			if (child ) 
 			{
-				gchar *childname = Accessible_getName (child);
+				gchar *childname;
+				gchar *spi_childname = Accessible_getName (child);
+
+				if (spi_childname && (strlen (spi_childname) > 0))  
+				{
+					childname = g_strdup (spi_childname);
+				}
+				else 
+				{
+					childname = gok_spy_concatenate_child_names (child, depth);
+				}
 				if (childname && (strlen (childname) > 0)) 
 				{
-					if (tmp)
+					if (tmp) 
 					{
 						name = g_strconcat (tmp, "", childname, NULL);
 						g_free (tmp);
 					}
-					else
+					else 
 					{
 						name = g_strdup (childname);
 					}
-					SPI_freeString (childname);
 					tmp = name;
 				}
+
+				SPI_freeString (spi_childname);	
+				g_free (childname);
 			}
 		}
 	}
@@ -1397,12 +1416,13 @@ gok_spy_resolve_namesakes (GSList *list)
 	/* for every node in the given list... */
 	while (list && list->next) {
 		AccessibleNode *node = list->data;
+		GSList *lp = list;
 		if (node && node->pname) 
 		{
 		    strncpy (name, node->pname, 63);
 		}
 		/* for every node with the same name as the current node... */
-		while (list && (namesake = gok_spy_node_get_node_named ( list->next, name )) 
+		while (lp && (namesake = gok_spy_node_get_node_named ( lp->next, name )) 
 				!= NULL) 
 		{
 			if (!distinguished_source) {
@@ -1410,7 +1430,7 @@ gok_spy_resolve_namesakes (GSList *list)
 				gok_spy_distinguish_node_name ( (AccessibleNode *) list->data );
 			}
 			gok_spy_distinguish_node_name ( (AccessibleNode *) namesake->data );
-			list = list->next;
+			lp = lp->next;
 		}
 		distinguished_source = FALSE;	
 		if (list) list = list->next;
@@ -1633,7 +1653,7 @@ GSList* gok_spy_append_node (GSList* nodes,
 		 * is the dominant user activity, this is especially appropriate.
 		 */
 		if (pName == NULL && role == SPI_ROLE_TABLE_CELL) {
-			pName = gok_spy_concatenate_child_names (pAccessible);
+			pName = gok_spy_concatenate_child_names (pAccessible, 0);
 		}
 		else if (pName == NULL && role == SPI_ROLE_SCROLL_BAR) {
 		    states = Accessible_getStateSet (pAccessible);
@@ -1687,16 +1707,17 @@ GSList* gok_spy_append_node (GSList* nodes,
 			if (caption) 
 			{
 				char *caption_string = Accessible_getName (caption);
-				if (caption_string) 
+				if (caption_string && strlen (caption_string)) 
 				{
 					pName = g_strndup (caption_string, 20);
+				        gok_log ("Table caption: %s", caption_string);
 					SPI_freeString (caption_string);
 				}
 			}
 			if (pName == NULL && AccessibleTable_getNColumns (table) == 1) 
 			{
 				char *desc = AccessibleTable_getColumnDescription (table, 0);
-				if (desc) 
+				if (desc && strlen (desc)) 
 				{
 					pName = g_strndup (desc, 20);
 					SPI_freeString (desc);
@@ -1708,8 +1729,9 @@ GSList* gok_spy_append_node (GSList* nodes,
 					if (header) 
 					{
 						desc = Accessible_getName (header);
-						if (desc) 
+						if (desc && strlen (desc)) 
 						{
+						        gok_log ("Table col header: %s", desc);
 							pName = g_strndup (desc, 20);
 							SPI_freeString (desc);
 						}
@@ -1727,6 +1749,8 @@ GSList* gok_spy_append_node (GSList* nodes,
 		if (pName != NULL) {
 			gok_log("we have a name: [%s]",pName);
 			/* create a new node */
+			if (*g_strstrip (pName) == 0 || strlen (pName) == 0) 
+			    g_warning ("EMPTY Node name: [%s]", pName);
 			nodes = gok_spy_add_node (nodes, pAccessible, flags, 0, pName);
 		}
 		/* if we decide to allow access to nameless menus ...
@@ -1793,6 +1817,8 @@ GSList* gok_spy_append_node (GSList* nodes,
 				}
 			}
 			if (pName != NULL) {
+			        if (*g_strstrip (pName) == 0 || strlen (pName) == 0) 
+				    g_warning ("EMPTY Node name: [%s]", pName);
 				gok_log ("we have a name: [%s]",pName);
 				nodes = gok_spy_add_node (nodes, pAccessible, flags, 0, pName);
 			}
@@ -2385,8 +2411,8 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 				if (non_context_flags.value) 
 				{
 				    _priv_ui_nodes = gok_spy_append_node (_priv_ui_nodes, child, flags);
-				    /* special case for notebook tabs, whose children should also appear */
-  				    if (role == SPI_ROLE_PAGE_TAB)
+				    /* special case for notebook tabs and expanders, whose children should also appear */
+  				    if (role == SPI_ROLE_PAGE_TAB || role == SPI_ROLE_TOGGLE_BUTTON)
   				    {
   					ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags, context_flags).value;
   				    }
