@@ -483,6 +483,26 @@ comparebasenames (gconstpointer a, gconstpointer b)
 	return returncode;
 }
 
+void 
+gok_main_center_corepointer (GtkWidget *window)
+{
+    Display *display;
+    GdkWindow *root;
+    GdkScreen *screen;
+    gint screen_width, screen_height;
+
+    window = gok_main_get_main_window ();
+    display = GDK_WINDOW_XDISPLAY (window->window);
+    screen = gdk_drawable_get_screen (window->window);
+    root = gdk_screen_get_root_window (screen);
+    screen_width = gdk_screen_get_width (screen);
+    screen_height = gdk_screen_get_height (screen);
+
+    XWarpPointer (display, None, 
+		  GDK_WINDOW_XWINDOW (root),
+		  0, 0, 0, 0, screen_width/2, screen_height/2);
+}
+
 /**
 * gok_main_open
 * @argc: ignored
